;;;   PNTSET.LSP v1.0
;;;   Josh Crawford 102700,1265 (search key JLISP)
;;;   January-September, 1995
;;;
;;;   PNTSET.LSP v1.0 is freeware.
;;;   Permission to use, copy, modify, and distribute this software and any
;;;   associated documentation for any purpose and without fee is granted on
;;;   the condition that this disclaimer header is INTACT.  Contact Josh
;;;   Crawford 102700,1265 with any improvements, modifications or errors.
;;;   Document any changes before posting on a forum, BBS or redistributing
;;;
;;;   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
;;;   ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE ARE HEREBY
;;;   DISCLAIMED.
;;;
;;;   THIS PROGRAM HAS BEEN USED ON r12 FOR DOS <ONLY>.  NO TESTING HAS BEEN
;;;   DONE ON ANY OTHER PLATFORM OR RELEASE.  (JC - 9/22/95)
;;;
;;;--------------------------------------------------------------------------
;;;   Contains a set of 6 point return functions and 2 angle return functions.
;;;   
;;;   I often need to find a point that can't be obtained with regular osnaps.
;;;   So rather than do the extra work of construction lines, I developed these
;;;   reference funtions similar to AutoCAD's .xy point filters.  The functions
;;;   return points (or angles) in several manners:
;;;
;  Label/Function
;;;   .A/(angl) - Returns an angle from or added to two points, a text or line 
;;;               entity. The text can be an attribute. When a line is picked,
;;;               there is a choice between the angle (beg-end) and it's 
;;;               opposite (end-beg).  The last prompt allows the user to add
;;;               to the selected angle (pos or neg).
;;;
;;;   .B/(bsct) - Bisect an angle (acute or obtuse) given by three points.
;;;               If (angl) is not being used transparently, an option to draw
;;;               a line representing the bisector is available.
;;;
;;;   .F/(frac) - Returns a point that is a fraction (as entered by user using
;;;               reals or integers) of the distance between the reference 
;;;               points picked by the user.  The numerator and denominator are
;;;               entered seperately. Can also use fractions like 3.5 over 1.25.
;;;
;;;   .I/(ints) - Returns a point that's the apparent intersection of two lines,
;;;               a line and two points or 4 points. This routine works good but
;;;               is very messy and not very robust.
;;;
;;;   .M/(midp) - Returns a point midway between two points.
;;;
;;;   .O/(oend) - Returns a point equal to the opposite end of the pick point on
;;;               a line.
;;;
;;;   .P/(polr) - Returns a point from polar coord. data (base,angle,distance)
;;;               repeating until the user returns a blank entry.
;;;
;;;   .R/(rect) - Returns a point by rectangular coordinates (absolute or
;;;               relative) repeating until the user returns a blank entry.
;;;
;;;   I use these functions in a 2D (r12) environment so I don't know how they 
;;;   act in 3D but they shouldn't be that difficult to modify if necessary.
;;;   PNT-POP0.mnu contains the POP0 menu that I use to transparently load/run
;;;   these functions. The initial selection of a function from the POP0 menu 
;;;   loads all of the functions.  If you like this program, come up with
;;;   improvements or discover errors, please let me know me.
;;;                                                     Josh Crawford
;;;                                                     102700,1265
;;;--------------------------------------------------------------------------

(setq PNTSET 1)
(prompt "\n>>Loading PNTSET v1.0 Freeware from JLisp /")

(prompt "\010|")
;****************************************************************
(defun errtrap (s)
 (if (/= s "Function cancelled")(princ (strcat "\nError: " s)))
 (setq *error* olderr)
 (princ)
)

(prompt "\010/")
;****************************************************************
;FROM AI_UTILS.LSP - PRODUCES A BEEP
(defun ai_beep ( / f)
 (write-line "\007" (setq f (open "CON" "w")))
 (setq f (close f))
 (princ)
)

(prompt "\010-")
;****************************************************************
; .A - (angl)
;RETURNS AN ANGLE FROM OR ADDED TO TWO POINTS, LINE OR TEXT ENTITY
(defun angl (/ au base nextpoint ang ang2 angle1 angle2)

 (defun getlineang (/ang ang-opp)
  (setq line-data (entget (car (entsel "\nSelect Line: "))))
  (setq ang (angle 
   (trans (cdr (assoc 10 line-data)) 0 1)(trans (cdr (assoc 11 line-data)) 0 1)))
  (setq ang-opp (+ ang pi))
  (princ (strcat "\nSelected: " (angtos ang) "  Opposite: " (angtos ang-opp)))
  (initget "Opposite Selected")
  (setq choice (getkword "\nOpposite/<Selected>: "))
  (if (= choice "Opposite") ang-opp ang)
 )

 (defun gettextang (/ text-ent)
  (while (not text-ent)(setq text-ent (entsel "\nSelect Text: ")))
  (setq text-data (entget (car text-ent)))
  (cdr (assoc 50 text-data))
 )

 (setq au (getvar "auprec"))
 (initget "Line Text")
 (setq base (getpoint "\n>>/Line/Text/<Pick base point>: "))
 (cond
  ((= base "Line")(setq ang (getlineang)))
  ((= base "Text")(setq ang (gettextang)))
  (t (progn
      (setq nextpoint (getpoint base "\n>>Pick second point: "))
      (setq ang (angle base nextpoint))
     )
  )
 )
 (setq angle1 (angtos ang 1 au) angle2 (angtos ang 1 14))
 (princ (Strcat "\n>>Angle: " angle1 "   -OR-   " angle2))
 (initget "Add")
 (setq do_add (getkword (strcat "\n>>Add/<" angle1 ">: ")))
 (if (= do_add "Add")
  (progn
   (initget 1)
   (setq ang2 (getangle (strcat "\n>> " angle1 " plus : ")))
   (setq ang (+ ang ang2))
   (setq angle1 (angtos ang 1 au) angle2 (angtos ang 1 14))
   (princ (Strcat "\n>>Angle: " angle1 "   -OR-   "))
   (princ angle2)
  )
  (princ angle2)
 )
)

(prompt "\010\\")
;****************************************************************
; .B - (bsct)
;BISECT ANGLE GIVEN BY VERTEX AND TWO POINTS OR ANGLES.
(defun bsct ( / vertex end1 end2 ang1 ang2 choice size)
 (setq olderr *error* *error* errtrap )
 (initget 1)
 (setq vertex (getpoint "\n>>Vertex point: "))
 (initget 1)
 (setq end1 (getangle vertex "\n>>First angle: "))
 (initget 1)
 (setq end2 (getangle vertex "\n>>Second angle: "))
 (setq ang1 (abs (- end1 end2))
       ang2 (- (* 2 pi) ang1))
 (if (< ang1 ang2)
  (setq acute-ang ang1 obtuse-ang ang2)
  (setq acute-ang ang2 obtuse-ang ang1)
 )
 (if (= (rem (+ acute-ang end1) (* 2 pi)) end2)
  (setq acute-bisect (+ (/ acute-ang 2.0) end1))
  (setq acute-bisect (+ (/ acute-ang 2.0) end2))
 )
 (setq obtuse-bisect (+ acute-bisect pi))
 (prompt (strcat "\n>>Acute angle:  " (angtos acute-ang)
                 " --- Obtuse angle:  " (angtos obtuse-ang)
                 "\n>>Acute bisector: " (angtos acute-bisect)
                 " --- Obtuse bisector: " (angtos obtuse-bisect)))
 (setq size (* 0.1 (getvar "viewsize")))
 (grdraw vertex (polar vertex acute-bisect size) -1 1)
 (if (= (getvar "cmdactive") 0)
  (progn
   (initget "Acute Obtuse Line")
   (setq choice (getkword "\n>>Line/Obtuse/<Acute> bisector: "))
  )
  (progn
   (initget "Acute Obtuse")
   (setq choice (getkword "\n>>Obtuse/<Acute> bisector: "))
  )
 )
 (setq *error* olderr)
 (grdraw vertex (polar vertex acute-bisect size) -1 1)
 (cond
  ((= choice "Line")
     (progn
       (command "line" vertex (polar vertex acute-bisect size) "")
       (princ)
     )
   )
  ((= choice "Obtuse") (angtos obtuse-bisect (getvar "aunits") 14))
  (t                   (angtos acute-bisect (getvar "aunits") 14))
 )
)

(prompt "\010|")
;****************************************************************
; .F - (frac)
;RETURNS A POINT THAT IS A FRACTION (AS ENTERED BY USER USING
;REALS OR INTEGERS ie 2 & 3.5 or 8.345 & 1.75) OF THE DISTANCE
;BETWEEN THE REFERENCE POINTS PICKED BY THE USER.
(DEFUN frac (/ base nextpoint ang dist num den frac olderr)
  (setq olderr *error* *error* errtrap)
  (setq base (getpoint "\n>>First point reference point: "))
  (setq nextpoint (getpoint base ">>Second point reference point: "))
  (grdraw base nextpoint -1 1)
  (setq ang (angle base nextpoint))
  (setq dist (distance base nextpoint))
  (prompt (strcat "\n>>Distance between reference points: " (rtos dist)))
  (initget 2)
  (setq num (getreal "\n>>Numerator<1.0>: "))
  (if (not num)(setq num 1.0))
  (initget 2)
  (setq den (getreal "\n>>Denominator<2.0>: "))
  (if (not den)(setq den 2.0))
  (setq frac (* dist (/ num den)))
  (grdraw base nextpoint -1 1)
  (setq *error* olderr)
  (if (> (getvar "osmode") 0)(command "_non"))
  (polar base ang frac)
)

(prompt "\010/")
;****************************************************************
; .I - (ints)
;RETURNS A POINT THAT IS THE APPARENT INTERSECTION OF TWO LINES,
;A LINE AND TWO POINTS OR 4 POINTS. THIS ROUTINE IS MESSY BUT WORKS
;DECENTLY ENOUGH. ONE PROBLEM IS PICKING A STRAIGHT POLYLINE SEGMENT
;AND THEN NOT HAVING THE POINT OPTION AFTER THE "LINE" WARNING MESSAGE.
;I'M LEAVING IT ALONE UNTIL I, OR SOMEONE ELSE, CAN COME BACK TO IT
;WITH A FRESH APPROACH THAT DOES NOT MAKE (INTS) GET MORE DISJOINTED.
(defun ints ( / a b c d phntm check olderr plist)
 (setq olderr *error* *error* errtrap )
 (setq check nil plist nil)
 (while (= check nil)
  (addpnts "first")
  (addpnts "second")
  (setq a (car plist) b (cadr plist) c (caddr plist) d (cadddr plist))
  (setq check (inters a b c d nil))
  (if (= check nil)
   (progn
    (prompt "\n>>Lines/Points are PARALLEL!")
    (ai_beep)
    (setq plist nil)
   )
  )
 )
 (setq *error* olderr)
 (if (= phntm "set") (grdraw (trans a 0 1) (trans b 0 1) -1 1))
 (if (> (getvar "osmode") 0)(command "_non"))
 (trans (inters a b c d nil) 0 1)
)

;PROMPT FOR A LINE UNTIL ONE IS FOUND
 (defun getline (msg / l)
  (setq l (car (entsel (strcat "\n>>Pick " msg " intersecting line " ))))
  (while (/= (cdr (assoc 0 (entget l))) "LINE")
   (prompt "\n>>Entity is not a LINE!")
   (ai_beep)
   (setq l (car (entsel (strcat "\n>>Pick " msg " intersecting line " ))))
  )
  l
 )

;EXTRACT LINE ENDS
 (defun set_a-b (entname)
  (setq a (cdr (assoc 10 (entget entname))))
  (setq b (cdr (assoc 11 (entget entname))))
 )

;POINT SELECTION CHOICE
 (defun addpnts (msg / choice a b lineone)
  (initget "Points Line")
  (setq choice (entsel (strcat "\n>>Select " msg " intersecting leg by Points/<Line>: ")))
  (cond
   ((= choice "Points")
     (initget 1)
     (setq a (getpoint "\n>>First point: "))
     (initget 1)
     (setq b (getpoint a "\n>>Second point: "))
     (if (not phntm)(progn (setq phntm "set")(grdraw a b -1 1)))
     (setq a (trans a 1 0) b (trans b 1 0))
   )
   ((or (= choice "Line")(= choice nil))
     (setq lineone (getline msg))
     (set_a-b lineone)
   )
   ((= (type (car choice)) 'ENAME)
     (if (= "LINE" (cdr (assoc 0 (entget (car choice)))))
      (progn
       (set_a-b (car choice))
      )
      (progn
       (prompt "\n>>Entity is not a LINE!")
       (ai_beep)
       (setq lineone (getline msg))
       (set_a-b lineone)
      )
    )
   )
  )
  (setq plist (append plist (list a) (list b)))
 )

(prompt "\010-")
;****************************************************************
; .M - (midp)
;RETURNS A POINT MIDWAY BETWEEN TWO POINTS.
(DEFUN midp (  / base nextpoint ang dist olderr )
 (setq olderr *error* *error* errtrap )
 (setq base (getpoint "\n>>First reference point: "))
 (setq nextpoint (getpoint base "\n>>Second reference point: "))
 (setq ang (angle base nextpoint))
 (setq dist (distance base nextpoint))
 (setq *error* olderr)
 (if (> (getvar "osmode") 0)(command "_non"))
 (polar base ang (/ dist 2))
)

(prompt "\010\\")
;****************************************************************
; .O - (oend)
;RETURNS A POINT EQUAL TO THE OPPOSITE END OF THE PICK POINT ON A LINE.
(defun oend ( / ent data check bgn_line end_line pick_pt
                    olderr )
 (setq *error* errtrap )
 (setq ent (entsel "\n>>Select line near the endpoint: " ))
 (setq data (entget (car ent)))
 (setq check nil)
 (while (not check)
  (if (= (cdr (assoc 0 data)) "LINE")(setq check "OK"))
  (if (not check)
   (progn
    (prompt "\n>>Entity is not a LINE!")
    (ai_beep)
    (setq ent (entsel "\n>>Select line near the endpoint:"))
    (setq data (entget (car ent)))
   )
  )
 )
 (setq bgn_line (cdr (assoc 10 data)))
 (setq end_line (cdr (assoc 11 data)))
 (setq pick_pt (trans (car (cdr ent)) 1 0))
 (setq *error* olderr)
 (if (> (getvar "osmode") 0)(command "_non"))
 (if (> (distance bgn_line pick_pt) (distance end_line pick_pt))
  (trans bgn_line 0 1)
  (trans end_line 0 1)
 )
)

(prompt "\010|")
;****************************************************************
; .P - (polr)
;RETURNS A POINT FROM POLAR COORDINATE DATA (BASE,ANGLE,DISTANCE)
;REPEATING UNTIL THE USER RETURNS A BLANK ENTRY.
(defun polr ( / base ang dist dist1 dist2 lastang lastbase
                    olderr )
 (setq olderr *error* *error* errtrap)
 (initget 1)
 (setq base (getpoint "\n>>Base: "))
 (initget "Points" 1)
 (setq ang (getangle base "\n>>Points/<Angle>: "))
 (while ang
  (if (eq ang "Points")
   (progn
    (prompt "\n>>First point: ")
    (setq ang (getangle))
   )
  )
  (initget "Points" 1)
  (setq dist (getdist base "\n>>Points/<Distance>: "))
  (if (eq dist "Points")
   (progn
    (setq dist1 (getpoint "\n>>First point: "))
    (setq dist2 (getpoint dist1 "\n>>Second point: "))
    (setq dist (distance dist1 dist2))
   )
  )
  (setq lastbase base
        base (polar base ang dist)
        lastang ang
        ang (getangle base "\n>>Points/<Angle>: ")
  )
 )
 (setq *error* olderr)
 (if (> (getvar "osmode") 0)(command "_non"))
 (polar lastbase lastang dist)
)

(prompt "\010/")
;****************************************************************
; .R - (rect)
;RETURNS A POINT BY RECTANGULAR COORDINATES (ABSOLUTE OR RELATIVE)
;REPEATING UNTIL THE USER RETURNS A BLANK ENTRY.
(defun rect (/ base nextpoint olderr )
 (setq olderr *error* *error* errtrap )
 (setq base (getpoint "\n>>Base point: "))
 (setq nextpoint (getpoint base "\n>>Reference point: "))
 (while nextpoint
  (setq base nextpoint)
  (setq nextpoint (getpoint base "\n>>Reference point: "))
 )
 (setq *error* olderr)
 (if (> (getvar "osmode") 0)(command "_non"))
 base
)

;****************************************************************
(prompt "\010Complete\n")
(princ)


